@@ -30,11 +30,10 @@
 #ifdef _WIN32
 #define NOGDI
 #endif
-#include <libusb-1.0/libusb.h>
+#include <libusb.h>
 #endif
 
-#include <libdivecomputer/atomics_cobalt.h>
-
+#include "atomics_cobalt.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "checksum.h"
@@ -76,6 +75,7 @@ static const dc_device_vtable_t atomics_cobalt_device_vtable = {
 	NULL, /* write */
 	NULL, /* dump */
 	atomics_cobalt_device_foreach, /* foreach */
+	NULL, /* timesync */
 	atomics_cobalt_device_close /* close */
 };
 
