@@ -255,22 +255,18 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	// Read the logbook type
-	rc = shearwater_common_identifier (&device->base, buffer, ID_RDBI);
+	rc = shearwater_common_identifier (&device->base, buffer, ID_LOGUPLOAD);
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR (abstract->context, "Failed to read the logbook type.");
 		dc_buffer_free (buffer);
 		dc_buffer_free (manifests);
 		return rc;
 	}
-	unsigned int base_addr = array_uint_be (dc_buffer_get_data (buffer), dc_buffer_get_size (buffer));
+	unsigned int base_addr = array_uint32_be (dc_buffer_get_data (buffer) + 1);
 	INFO(abstract->context, "RDBI command completed with %d bytes, evaluated as %08x", (int) dc_buffer_get_size (buffer), base_addr);
-	base_addr &= 0xFF000000u;
 	switch (base_addr) {
-	case 0xDD000000: // Predator or Predator-Like Format
-		// on a Predator, use the old format, otherwise use the Predator-Like Format (what we called Petrel so far)
-		if (model != PREDATOR)
-			base_addr = 0xC0000000u;
-		break;
+	case 0xDD000000: // Predator - we shouldn't get here, we could give up or we can try 0xC0000000
+	case 0xC0000000: // Predator-Like Format (what we used to call the Petrel format)
 	case 0x90000000: // some firmware versions supported an earlier version of PNF without final record
 		// use the Predator-Like Format instead
 		base_addr = 0xC0000000u;
@@ -279,11 +275,10 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 		// that's the correct address
 		break;
 	default: // unknown format
-		// use the defaults for the models
-		if (model >= TERIC)
-			base_addr = 0x80000000u;
-		else
-			base_addr = 0xC0000000u;
+		ERROR (abstract->context, "Unknown logbook download format.");
+		dc_buffer_free (buffer);
+		dc_buffer_free (manifests);
+		return DC_STATUS_DATAFORMAT;
 	}
 
 	// Read the manifest pages
@@ -389,11 +384,6 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 
 		offset += RECORD_SIZE;
 	}
-	// send the "graceful exit" instruction
-	// I don't think we care about the return value of this call - this is just trying to be nice to the device
-	unsigned char command[4] = { 0x2e, 0x90, 0x20, 0x00 };
-	rc = shearwater_common_command(&device->base, command, 4);
-	DEBUG(abstract->context, "Sent graceful exit command, rc=%d", rc);
 
 	// Update and emit a progress event.
 	progress.current = NSTEPS * current;
